@@ -16,7 +16,8 @@ import grp
 import shutil
 import tempfile
 
-from utils import get_shell_output2
+from utils import get_shell_output, get_shell_output2, encode, decode, \
+                  ask_convert_invalid_encoding_filename
 
 
 ########################################################################
@@ -24,14 +25,15 @@ from utils import get_shell_output2
 # File Type:    dir, link to directory, link, nlink, char dev,
 #               block dev, fifo, socket, executable, file
 (FTYPE_DIR, FTYPE_LNK2DIR, FTYPE_LNK, FTYPE_NLNK, FTYPE_CDEV, FTYPE_BDEV,
- FTYPE_FIFO, FTYPE_SOCKET, FTYPE_EXE, FTYPE_REG) = xrange(10)
+ FTYPE_FIFO, FTYPE_SOCKET, FTYPE_EXE, FTYPE_REG, FTYPE_UNKNOWN) = xrange(11)
 
 FILETYPES = { FTYPE_DIR: (os.sep, 'Directory'),
               FTYPE_LNK2DIR: ('~', 'Link to Directory'),
               FTYPE_LNK: ('@', 'Link'), FTYPE_NLNK: ('!', 'No Link'),
               FTYPE_CDEV: ('-', 'Char Device'), FTYPE_BDEV: ('+', 'Block Device'),
               FTYPE_FIFO: ('|', 'Fifo'), FTYPE_SOCKET: ('#', 'Socket'),
-              FTYPE_EXE: ('*', 'Executable'), FTYPE_REG: (' ', 'File') }
+              FTYPE_EXE: ('*', 'Executable'), FTYPE_REG: (' ', 'File'),
+              FTYPE_UNKNOWN: ('?', 'Unknown') }
 
 (FT_TYPE, FT_PERMS, FT_OWNER, FT_GROUP, FT_SIZE, FT_MTIME) = xrange(6)
 
@@ -50,7 +52,7 @@ except AttributeError:
     def get_rdev(f):
         """'ls -la' to get mayor and minor number of devices"""
         try:
-            buf = os.popen('ls -la %s' % f).read().split()
+            buf = get_shell_output('ls -la %s' % encode(f))
         except:
             return 0
         else:
@@ -68,13 +70,9 @@ else:
 
 def __get_size(f):
     """return the size of the directory or file via 'du -sk' command"""
-
-    buf = get_shell_output2('du -sk \"%s\"' % f)
-#     return int(buf.split()[0])*1024 if buf else 0
-    if buf:
-        return int(buf.split()[0]) * 1024
-    else:
-        return 0
+    
+    buf = get_shell_output2('du -sk \"%s\"' % encode(f))
+    return (buf==None) and 0 or int(buf.split()[0])*1024
 
 
 def get_realpath(path, filename, filetype):
@@ -102,11 +100,9 @@ def join(directory, f):
     return os.path.join(directory, f)
 
 
-def __get_filetype(f):
+def __get_filetype(lmode, f):
     """get the type of the file. See listed types above"""
 
-    f = os.path.abspath(f)
-    lmode = os.lstat(f)[stat.ST_MODE]
     if stat.S_ISDIR(lmode):
         return FTYPE_DIR
     if stat.S_ISLNK(lmode):
@@ -133,12 +129,16 @@ def __get_filetype(f):
         return FTYPE_REG       # if no other type, regular file
 
 
-def get_fileinfo(f, pardir_flag = False, show_dirs_size = False):
-    """return information about a file in next format:
+def get_fileinfo(f, pardir_flag=False, show_dirs_size=False):
+    """return information about a file, with format:
     (filetype, perms, owner, group, size, mtime)"""
 
-    st = os.lstat(f)
-    typ = __get_filetype(f)
+    f = os.path.abspath(f)
+    try:
+        st = os.lstat(f)
+    except OSError:
+        return (FTYPE_UNKNOWN, 0, 'root', 'root', 0, 0)
+    typ = __get_filetype(st[stat.ST_MODE], f)
     if typ in (FTYPE_DIR, FTYPE_LNK2DIR) and not pardir_flag and show_dirs_size:
         size = __get_size(f)
     elif typ in (FTYPE_CDEV, FTYPE_BDEV):
@@ -161,6 +161,20 @@ def get_fileinfo(f, pardir_flag = False, show_dirs_size = False):
             size, st[stat.ST_MTIME])
 
 
+def get_fileinfo_extended(f):
+    """return additional information about a file, with format:
+    (num_links, uid, gid, atime, mtime, ctime, inode, dev)"""
+
+    try:
+        st = os.lstat(f)
+    except OSError:
+        return (0, 0, 0, 0, 0, 0, 0, 0)
+    else:
+        return (st[stat.ST_NLINK], st[stat.ST_UID], st[stat.ST_GID],
+                st[stat.ST_ATIME], st[stat.ST_MTIME], st[stat.ST_CTIME],
+                st[stat.ST_DEV], st[stat.ST_INO])
+
+
 def perms2str(p):
     permis = ['x', 'w', 'r']
     perms = ['-'] * 9
@@ -183,7 +197,7 @@ def get_fileinfo_dict(path, filename, filevalues):
     res['filename'] = filename
     typ = filevalues[FT_TYPE]
     res['type_chr'] = FILETYPES[typ][0]
-    if typ == (FTYPE_CDEV, FTYPE_BDEV):
+    if typ in (FTYPE_CDEV, FTYPE_BDEV):
         # HACK: it's too time consuming to calculate all files' rdevs
         #       in a directory, so we just calculate needed ones here
         #       at show time
@@ -219,11 +233,13 @@ def get_fileinfo_dict(path, filename, filevalues):
     return res
 
 
-def get_dir(path, show_dotfiles = 1):
+def get_dir(path, show_dotfiles=1):
     """return a dict whose elements are formed by file name as key
     and a (filetype, perms, owner, group, size, mtime) tuple as value"""
 
-    path = os.path.normpath(path)
+    # bug in python: os.path.normpath(u'/') returns str instead of unicode,
+    #                so convert to unicode anyway
+    path = decode(os.path.normpath(path))   
     files_dict = {}
     if path != os.sep:
         files_dict[os.pardir] = get_fileinfo(os.path.dirname(path), 1)
@@ -231,6 +247,11 @@ def get_dir(path, show_dotfiles = 1):
     if not show_dotfiles:
         files_list = [f for f in files_list if f[0] != '.']
     for f in files_list:
+        if not isinstance(f, unicode):
+            newf = decode(f)
+            if ask_convert_invalid_encoding_filename(newf):
+                convert_filename_encoding(path, f, newf)
+            f = newf
         files_dict[f] = get_fileinfo(os.path.join(path, f))
     return len(files_dict), files_dict
 
@@ -253,7 +274,7 @@ def get_groups():
     return [e[0] for e in grp.getgrall()]
 
 
-def set_perms(f, perms):
+def set_perms(f, perms, recursive=False):
     """set permissions to a file"""
     ps, i = 0, 8
     for p in perms:
@@ -272,12 +293,18 @@ def set_perms(f, perms):
                 ps += 2 * 8 ** 3
         i -= 1
     try:
-        os.chmod(f, ps)
+        if recursive:
+            pc = PathContents([f], os.path.dirname(f))
+            for e, s in pc.iter_walk():
+                if not os.path.isdir(e):
+                    os.chmod(e, ps)
+        else:
+            os.chmod(f, ps)
     except (IOError, os.error), (errno, strerror):
         return (strerror, errno)
 
 
-def set_owner_group(f, owner, group):
+def set_owner_group(f, owner, group, recursive=False):
     """set owner and group to a file"""
     try:
         owner_n = pwd.getpwnam(owner)[2]
@@ -288,59 +315,50 @@ def set_owner_group(f, owner, group):
     except:
         group_n = int(group)
     try:
-        os.chown(f, owner_n, group_n)
+        if recursive:
+            pc = PathContents([f], os.path.dirname(f))
+            for e, s in pc.iter_walk():
+                os.chown(e, owner_n, group_n)
+        else:
+            os.chown(f, owner_n, group_n)
     except (IOError, os.error), (errno, strerror):
         return (strerror, errno)
 
-def get_fs_info():
-    """return a list containing the info returned by 'df -k', i.e,
-    file systems size and occupation, in Mb. And the filesystem type:
-    [dev, size, used, available, use%, mount point, fs type]"""
 
+def get_mount_points():
+    """return system mount points as list of (mountpoint, device, fstype).
+    Compatible with linux and solaris"""
     try:
-        buf = os.popen('df -k').readlines()
+        buf = get_shell_output('mount')
     except (IOError, os.error), (errno, strerror):
         return (strerror, errno)
-    fs = []
-    for l in buf:
-        if l[0] == os.sep:
-            e = l.split()
-            if len(e) > 1:
-                e[1] = str(int(e[1]) / 1024)
-                e[2] = str(int(e[2]) / 1024)
-                e[3] = str(int(e[3]) / 1024)
-                e[4] = e[4]
-                e[5] = e[5]
-            else:
-                continue
-        elif l[0] == ' ':
-            t = l.split()
-            e.append(str(int(t[0]) / 1024))
-            e.append(str(int(t[1]) / 1024))
-            e.append(str(int(t[2]) / 1024))
-            e.append(t[3])
-            e.append(t[4])
-        else:
-            continue
-        fs.append(e)
+    if buf is None or buf == '':
+        return ('Can\t run "mount" command', 0)
+    lst = []
+    for e in buf.split('\n'):
+        es = e.split()
+        lst.append((es[2], es[0], es[4]))
+    # lst.sort(reverse=True) # python v2.4+
+    lst.sort()
+    lst.reverse()
+    return lst
 
-    # get filesystems type
-    if sys.platform[:5] == 'linux':
-        es = open('/etc/fstab').readlines()
-        fstype_pos = 2
-    elif sys.platform[:5] == 'sunos':
-        es = open('/etc/vfstab').readlines()
-        fstype_pos = 3
+
+def get_mountpoint_for_file(f):
+    # check mps
+    mps = get_mount_points()
+    for m, d, t in mps:
+        if f.find(m) != -1:
+            return (m, d, t)
     else:
-        es = []
-    for f in fs:
-        for e in es:
-            if e.find(f[5]) != -1:
-                f.append(e.split()[fstype_pos])
-                break
-        else:
-            f.append('unknown')
-    return fs
+        return ('/', '<unknown>', '<unknown>')
+
+
+def convert_filename_encoding(path, filename, newname):
+    curpath = os.getcwd()
+    os.chdir(path)
+    os.rename(filename, newname)
+    os.chdir(curpath)
 
 
 ########################################################################
@@ -424,23 +442,29 @@ def sort_dir(files_dict, sortmode, sort_mix_dirs, sort_mix_cases):
 ##### complete
 def complete(entrypath, panelpath):
     if not entrypath:
-        path = panelpath
+        path = panelpath + os.sep
+        base = ''
     elif entrypath[0] == os.sep:
         path = entrypath
+        base = os.path.dirname(entrypath)
     else:
         path = os.path.join(panelpath, entrypath)
+        base = os.path.dirname(entrypath)
     # get elements
-    if os.path.isdir(path):
-        basedir = path
-        fs = os.listdir(path)
-    else:
-        basedir = os.path.dirname(path)
-        start = os.path.basename(path)
-        try:
-            entries = os.listdir(basedir)
-        except OSError:
-            entries = []
-        fs = [f for f in entries if f.find(start, 0) == 0]
+    try:
+        if path.endswith(os.sep) and os.path.isdir(path):
+            basedir = path
+            fs = os.listdir(path)
+        else:
+            basedir = os.path.dirname(path)
+            start = os.path.basename(path)
+            try:
+                entries = os.listdir(basedir)
+            except OSError:
+                entries = []
+            fs = [f for f in entries if f.startswith(start)]
+    except (IOError, os.error), (errno, strerror):
+        fs = []
     # sort files with dirs first
     d1, d2 = [], []
     for f in fs:
@@ -451,12 +475,11 @@ def complete(entrypath, panelpath):
     d1.sort()
     d2.sort()
     d1.extend(d2)
-    return d1
+    return base, d1
 
 
 ########################################################################
 ##### actions
-# link
 def do_create_link(pointto, link):
     os.symlink(pointto, link)
 
@@ -476,108 +499,108 @@ def create_link(pointto, linkname):
         return (strerror, errno)
 
 
-# copy
-def do_copy(source, dest):
-    if os.path.islink(source):
-        dest = os.path.join(os.path.dirname(dest), os.path.basename(source))
+def copy_bulk(src, dest):
+    if os.path.isdir(src):
+        shutil.copytree(src, dest, symlinks=True)
+    elif os.path.isfile(src):
+        shutil.copy2(src, dest)
+
+
+def delete_bulk(path, ignore_errors=False):
+    if os.path.isdir(path):
+        shutil.rmtree(path, ignore_errors=ignore_errors)
+    elif os.path.isfile(path):
+        if ignore_errors:
+            try:
+                os.unlink(path)
+            except:
+                pass
+        else:
+            os.unlink(path)
+
+
+def do_copy(filename, basepath, dest, rename_dir=False, check_fileexists=True):
+    src = os.path.join(basepath, filename)   
+    if os.path.exists(dest) and os.path.isdir(dest):
+        if rename_dir:
+            filename = os.sep.join(filename.split(os.sep)[1:])
+        dest = os.path.join(dest, filename)
+    if os.path.exists(dest) and check_fileexists:
+        return os.path.basename(dest)
+    if os.path.islink(src):
         try:
-            do_create_link(os.readlink(source), dest)
+            do_create_link(os.readlink(src), dest)
         except (IOError, os.error), (errno, strerror):
             return (strerror, errno)
-    elif os.path.isdir(source):
+    elif os.path.isdir(src):
         try:
             os.mkdir(dest)
         except (IOError, os.error), (errno, strerror):
-            pass     # don't return if directory exists
+            if errno != 17:     # don't error if directory exists
+                return (strerror, errno)
         else:
-#             # copy mode, times, owner and group
-#             st = os.lstat(source)
-#             os.chown(dest, st[stat.ST_UID], st[stat.ST_GID])
-#             shutil.copymode(source, dest)
-#             shutil.copystat(source, dest)
-            pass
-        for f in os.listdir(source):
-            do_copy(os.path.join(source, f), os.path.join(dest, f))
-    elif source == dest:
-        raise IOError, (0, "Source and destination are the same file")
+            # copy mode, times, owner and group
+            try:
+                st = os.lstat(src)
+                os.chown(dest, st[stat.ST_UID], st[stat.ST_GID])
+                shutil.copymode(src, dest)
+                shutil.copystat(src, dest)
+            except (IOError, os.error), (errno, strerror):
+                pass
+    elif src == dest:
+        return ('Source and destination are the same file', 0)
     else:
-        shutil.copy2(source, dest)
+        if os.path.isfile(src): # stat.S_ISREG(os.lstat(src)[stat.ST_MODE])
+            try:
+                shutil.copy2(src, dest)
+            except (IOError, os.error), (errno, strerror):
+                return (strerror, errno)
+        else:
+            return ('Special files can\'t be copied or moved', 0)
 
 
-def copy(f, path, destdir, check_fileexists = True):
-    """ copy file / dir to destdir"""
-
-    fullpath = os.path.join(path, f)
-    if destdir[0] != os.sep:
-        destdir = os.path.join(path, destdir)
-    if os.path.isdir(destdir):
-        destdir = os.path.join(destdir, f)
-    if os.path.exists(destdir) and check_fileexists:
-        return os.path.basename(destdir)
+def do_rename(f, path, newname, check_fileexists=True):
+    src = os.path.join(path, f)
+    if newname[0] != os.sep:
+        dest = os.path.join(path, newname)
+    else:
+        dest = newname
+    if src == dest:
+        return ('Source and destination are the same file', 0)
+    if os.path.dirname(dest) != path:
+        return ('Can\'t rename to different directory', 0)
+    if os.path.isfile(dest) and check_fileexists:
+        return os.path.basename(newname)
     try:
-        do_copy(fullpath, destdir)
+        os.rename(src, dest)
     except (IOError, os.error), (errno, strerror):
         return (strerror, errno)
 
 
-# move
-def move(f, path, destdir, check_fileexists = True):
-    """delete file / dir"""
-
-    fullpath = os.path.join(path, f)
-    if destdir[0] != os.sep:
-        destdir = os.path.join(path, destdir)
-    if os.path.isdir(destdir):
-        destdir = os.path.join(destdir, f)
-    if os.path.exists(destdir) and check_fileexists:
-        return os.path.basename(destdir)
-    if os.path.dirname(fullpath) == os.path.dirname(destdir):
-        try:
-            os.rename(fullpath, destdir)
-        except (IOError, os.error), (errno, strerror):
-            return (strerror, errno)
-        return
+def do_backup(f, path, backup_ext):
+    src = os.path.join(path, f)
+    dest = os.path.join(path, f+backup_ext)
+    if os.path.exists(dest):
+        return ('File exists', 0)
     try:
-        do_copy(fullpath, destdir)
+        copy_bulk(src, dest)
     except (IOError, os.error), (errno, strerror):
-        try:
-            do_delete(destdir)
-        except (IOError, os.error), (errno, strerror):
-            return (strerror, errno)
         return (strerror, errno)
-    else:
-        try:
-            do_delete(fullpath)
-        except (IOError, os.error), (errno, strerror):
-            return (strerror, errno)
 
 
-# delete
 def do_delete(f):
-    if os.path.islink(f):
-        os.unlink(f)
-    elif os.path.isdir(f):
-        for f2 in os.listdir(f):
-            do_delete(os.path.join(f, f2))
-        os.rmdir(f)
-    else:
-        os.unlink(f)
-
-
-def delete(f, path):
-    """delete file / dir"""
-
-    fullpath = os.path.join(path, f)
     try:
-        do_delete(fullpath)
+        if os.path.islink(f):
+            os.unlink(f)
+        elif os.path.isdir(f):
+            os.rmdir(f)
+        else:
+            os.unlink(f)
     except (IOError, os.error), (errno, strerror):
         return (strerror, errno)
 
 
-# mkdir
 def mkdir(path, newdir):
-    """create directory"""
-
     fullpath = os.path.join(path, newdir)
     try:
         os.makedirs(fullpath)
@@ -585,4 +608,93 @@ def mkdir(path, newdir):
         return (strerror, errno)
 
 
+########################################################################
+##### PathContents
+class PathContents(object):
+    def __init__(self, fs, basepath=None):
+        """fs must be a list with relative path to files"""
+        if not isinstance(fs, list) or not len(fs) > 0:
+            raise TypeError, "fs must be a list of files"
+        if basepath is None:
+            basepath = os.getcwd()
+        self.basepath = os.path.abspath(basepath)
+        if not os.path.isdir(self.basepath):
+            raise TypeError, "basepath must be a valid directory or None"
+        self.__entries = []
+        self.__errors = []        
+        for f in fs:
+            f = os.path.join(self.basepath, f)
+            try:
+                if os.path.islink(f):
+                    self.__entries.append((f, 0))
+                else:
+                    self.__entries.append((f, os.path.getsize(f)))
+            except (IOError, os.error), (errno, strerror):
+                self.__errors.append((f, (strerror, errno)))
+            else:
+                if os.path.isdir(f) and not os.path.islink(f):
+                    try:
+                        self.__fill_contents(f)
+                    except UnicodeDecodeError:
+                        raise UnicodeError
+        self.length = len(fs)
+        self.tlength = len(self.__entries)
+        self.tsize = sum([f[1] for f in self.__entries]) or 1
+
+    def __fill_contents(self, path):
+        for root, dirs, files in os.walk(path, topdown=False, onerror=self.__on_error):
+            for d in dirs:
+                fullpath = os.path.join(root, d)
+                try:
+                    if os.path.islink(fullpath):
+                        self.__entries.append((fullpath, 0))
+                    else:
+                        self.__entries.append((fullpath, os.path.getsize(fullpath)))
+                except (IOError, os.error), (errno, strerror):
+                    self.__errors.append((fullpath, (strerror, errno)))
+            for f in files:
+                fullpath = os.path.join(root, f)
+                try:
+                    if os.path.islink(fullpath):
+                        self.__entries.append((fullpath, 0))
+                    else:
+                        self.__entries.append((fullpath, os.path.getsize(fullpath)))
+                except (IOError, os.error), (errno, strerror):
+                    self.__errors.append((fullpath, (strerror, errno)))
+
+    def __on_error(self, exc):
+        # print '[Error %d] %s: %s' % (exc.errno, exc.strerror, exc.filename)
+        fullpath = os.path.join(self.basepath, exc.filename)
+        self.__errors.append((fullpath, (exc.strerror, exc.errno)))
+
+    def __repr__(self):
+        return u'PathContents[Base:"%s" with %d entries (Total: %d items, %.2f KB)]' % \
+            (self.basepath, self.length, self.tlength, self.tsize/1024)
+
+    @property
+    def entries(self, reverse=False):
+        return sorted(self.__entries, reverse=reverse)
+        
+    @property
+    def errors(self):
+        return sorted(self.__errors)
+
+    def iter_walk(self, reverse=False):
+        for e in sorted(self.__entries, reverse=reverse):
+            yield e
+
+    def remove_files(self, fs):
+        new = []
+        length = self.length
+        for f, s in self.__entries:
+            if f in fs:
+                length -= 1
+            else:
+                new.append((f, s))
+        self.__entries = new
+        self.length = (length > 0) and length or 0
+        self.tlength = len(self.__entries)
+        self.tsize = sum([f[1] for f in self.__entries]) or 1
+        
+    
 ########################################################################
